#!/usr/bin/env python

"""This module remains for backwards compatibility reasons and will be
removed in PyDy 0.4.0."""

from pkg_resources import parse_version
import warnings

import pydy
from ..utils import PyDyDeprecationWarning
from .ode_function_generators import generate_ode_function as new_gen_ode_func


if parse_version(pydy.__version__) > parse_version('0.4.0'):
    msg = ("This module, 'pydy.codegen.code', is no longer supported. "
           "Please remove this module.")
    raise ValueError(msg)

warnings.warn("This module, 'pydy.codgen.code', is deprecated. The "
              "function 'generate_ode_function' can be found in the "
              "'pydy.codegen.ode_function_generator' module. "
              "'CythonGenerator' has been removed, use "
              "'pydy.codegen.cython_code.CythonMatrixGenerator' "
              "instead.",
              PyDyDeprecationWarning)


class CythonGenerator(object):
    def __init__(self, *args, **kwargs):
        warnings.warn("'CythonGenerator' has been removed, use "
                      "'pydy.codegen.cython_code.CythonMatrixGenerator' "
                      "instead.", PyDyDeprecationWarning)


def generate_ode_function(mass_matrix, forcing_vector, constants,
                          coordinates, speeds, specified=None,
                          generator='lambdify'):
    """Returns a numerical function which can evaluate the right hand side
    of the first order ordinary differential equations from a system
    described by:

    M(constants, coordinates) x' = F(constants, coordinates, speeds, specified)

    Parameters
    ----------
    mass_matrix : sympy.Matrix, shape(n,n)
        The symbolic mass matrix of the system. The rows should correspond
        to the coordinates and speeds.
    forcing_vector : sympy.Matrix, shape(n,1)
        The symbolic forcing vector of the system.
    constants : list of sympy.Symbol
        The constants in the equations of motion.
    coordinates : list of sympy.Function
        The generalized coordinates of the system.
    speeds : list of sympy.Function
        The generalized speeds of the system.
    specified : list of sympy.Function
        The specifed quantities of the system.
    generator : string, {'lambdify'|'theano'|'cython'}, optional
        The method used for generating the numeric right hand side.

    Returns
    -------
    evaluate_ode_function : function
        A function which evaluates the derivaties of the states.

    """
    warnings.warn("This function is deprecated and will be removed in PyDy "
                  "0.4.0. Use the the new 'generate_ode_function' in "
                  "'pydy.codegen.ode_function_generator'",
                  PyDyDeprecationWarning)

    return new_gen_ode_func(forcing_vector, coordinates, speeds, constants,
                            mass_matrix=mass_matrix, specifieds=specified,
                            generator=generator)



#!/usr/bin/env python

"""This module contains source code dedicated to generating C code from
matrices generated from sympy.physics.mechanics."""

import os
import itertools

import sympy as sm
import sympy.physics.mechanics as me
from sympy.printing.ccode import CCodePrinter

from ..utils import wrap_and_indent, find_dynamicsymbols


class CMatrixGenerator(object):
    """This class generates C source files that simultaneously numerically
    evaluate any number of SymPy matrices.

    """

    _c_header_template = """\
void evaluate(
{input_args}
{output_args}
             );
/*

{input_docstring}

*/"""

    _c_source_template = """\
#include <math.h>{header_include}

void evaluate(
{input_args}
{output_args}
             )
{{

{subexprs}

{outputs}

}}\
"""

    def __init__(self, arguments, matrices):
        """

        Parameters
        ==========
        arguments : sequences of sequences of SymPy Symbol or Function.
            Each of the sequences will be converted to input arrays in the C
            function. All of the symbols/functions contained in ``matrices``
            need to be in the sequences, but the sequences can also contain
            extra symbols/functions that are not contained in the matrices.
        matrices : sequence of SymPy.Matrix
            A sequence of the matrices that should be evaluated in the
            function. The expressions should contain only sympy.Symbol or
            sympy.Function that are functions of me.dynamicsymbols._t.

        """

        required_args = set()

        for matrix in matrices:
            # TODO : SymPy 0.7.4 does not have Matrix.free_symbols so we
            # manually compute them instead of calling:
            # required_args |= matrix.free_symbols
            required_args |= set().union(*[i.free_symbols for i in matrix])
            required_args |= find_dynamicsymbols(matrix)

        required_args.remove(me.dynamicsymbols._t)

        all_arguments = set(itertools.chain(*arguments))

        for required_arg in required_args:
            if required_arg not in all_arguments:
                msg = "{} is missing from the argument sequences."
                raise ValueError(msg.format(required_arg))

        self.matrices = matrices
        self.arguments = arguments

        self._generate_cse()
        self._generate_code_blocks()

    def _generate_cse(self):

        # This makes a big long list of every expression in all of the
        # matrices.
        exprs = []
        for matrix in self.matrices:
            exprs += matrix[:]

        # Compute the common subexpresions.
        gen = sm.numbered_symbols('pydy_')
        self.subexprs, simplified_exprs = sm.cse(exprs, symbols=gen)

        # Turn the expressions back into matrices of the same type.
        simplified_matrices = []
        idx = 0
        for matrix in self.matrices:
            num_rows, num_cols = matrix.shape
            length = num_rows * num_cols
            m = type(matrix)(simplified_exprs[idx:idx + length])
            simplified_matrices.append(m.reshape(num_rows, num_cols))
            idx += length

        self.simplified_matrices = tuple(simplified_matrices)

    def _generate_pydy_c_printer(self):
        """Returns a subclass of sympy.printing.CCodePrinter to print
        appropriate C array index calls for all of the symbols in the
        equations of motion.

        Examples
        --------

        >>> from sympy import symbols
        >>> from sympy.physics.mechanics import dynamicsymbols
        >>> from pydy.codegen.c_code import CMatrixGenerator
        >>> generator = CMatrixGenerator(...)
        >>> PyDyCCodePrinter = generator._generate_pydy_c_printer()
        >>> printer = PyDyCCodePrinter()
        >>> m = symbols('m') # m is the first constant in the EoMs
        >>> printer.doprint(m)
        input_0[0]
        >>> q = dynamicsymbols('q') # q is the second coordinate in the EoMs
        >>> printer.doprint(q)
        input_1[1]
        >>> F = dynamicsymbols('F') # F is the third specified in the EoMs
        >>> printer.doprint(F)
        input_3[2]

        """

        array_index_map = {}
        for i, arg_set in enumerate(self.arguments):
            for j, var in enumerate(arg_set):
                array_index_map[var] = r'input_{}[{}]'.format(i, j)

        class PyDyCCodePrinter(CCodePrinter):

            def _print_Function(self, e):
                if e in array_index_map.keys():
                    return array_index_map[e]
                else:
                    return super(PyDyCCodePrinter, self)._print_Function(e)

            def _print_Symbol(self, e):
                if e in array_index_map.keys():
                    return array_index_map[e]
                else:
                    return super(PyDyCCodePrinter, self)._print_Symbol(e)

        return PyDyCCodePrinter

    def comma_lists(self):
        """Returns a string output for each of the sequences of SymPy
        arguments."""

        comma_lists = []

        for i, arg_set in enumerate(self.arguments):
            comma_lists.append(', '.join([str(s) for s in arg_set]))

        return tuple(comma_lists)

    def _generate_code_blocks(self):
        """Writes the blocks of code for the C file."""

        printer = self._generate_pydy_c_printer()()

        self.code_blocks = {}

        lines = []
        for i, input_arg in enumerate(self.arguments):
            lines.append('double input_{}[{}],'.format(i, len(input_arg)))
        self.code_blocks['input_args'] = wrap_and_indent(lines, 14)

        lines = []
        for i, output_arg in enumerate(self.matrices):
            nr, nc = output_arg.shape
            lines.append('double output_{}[{}],'.format(i, nr * nc))
            self.code_blocks['output_args'] = \
                wrap_and_indent(lines, 14)[:-1]  # remove last comma

        lines = []
        for i, (input_arg, explan) in enumerate(zip(self.arguments,
                                                    self.comma_lists())):
            lines.append('input_{}[{}] : [{}]'.format(i, len(input_arg),
                                                      explan))
        self.code_blocks['input_docstring'] = wrap_and_indent(lines, 0)

        lines = []
        for var, expr in self.subexprs:
            var_str = printer.doprint(var)
            expr_str = printer.doprint(expr)
            lines.append('double {} = {};'.format(var_str, expr_str))
        self.code_blocks['subexprs'] = wrap_and_indent(lines)

        outputs = ''
        for i, output in enumerate(self.simplified_matrices):
            nr, nc = output.shape
            lhs = sm.MatrixSymbol('output_{}'.format(i), nr, nc)
            try:
                code_str = printer.doprint(output, lhs)
            except AttributeError:
                # The above fails in SymPy 0.7.4.1 because Matrix printing
                # isn't supported.
                code_lines = []
                for j, element in enumerate(output):
                    assignment = 'output_{}[{}]'.format(i, j)
                    code_lines.append(printer.doprint(element, assignment))
                code_str = '\n'.join(code_lines)
            outputs += wrap_and_indent(code_str.split('\n'))
            if i != len(self.simplified_matrices) - 1:
                outputs += '\n\n'  # space between each output

        self.code_blocks['outputs'] = outputs

    def doprint(self, prefix=None):
        """Returns a string each for the header and the source files.

        Parameters
        ==========
        prefix : string, optional
            A prefix for the name of the header file. This will cause an
            include statement to to be added to the source.

        """

        if prefix is not None:
            filling = {'header_include': '\n#include "{}.h"'.format(prefix)}
        else:
            filling = {'header_include': ''}

        filling.update(self.code_blocks)

        c_header = self._c_header_template.format(**filling)
        c_source = self._c_source_template.format(**filling)

        return c_header, c_source

    def write(self, prefix, path=None):
        """Writes a header and source file to disk.

        Parameters
        ==========
        prefix : string
            Two files will be generated: ``<prefix>.c`` and
            ``<prefix>.h``.

        """

        if path is None:
            path = os.getcwd()

        header, source = self.doprint(prefix=prefix)

        with open(os.path.join(path, prefix + '.h'), 'w') as f:
            f.write(header)

        with open(os.path.join(path, prefix + '.c'), 'w') as f:
            f.write(source)



#!/usr/bin/env python

import os
import sys
import signal
import socket
import webbrowser

# For python 2 and python 3 compatibility
if sys.version_info < (3, 0):
    from SimpleHTTPServer import SimpleHTTPRequestHandler
    from BaseHTTPServer import HTTPServer
else:
    from http.server import SimpleHTTPRequestHandler
    from http.server import HTTPServer
    raw_input = input

__all__ = ['Server']


class StoppableHTTPServer(HTTPServer):
    """
    Overrides BaseHTTPServer.HTTPServer to include a stop
    function.
    """

    def server_bind(self):
        HTTPServer.server_bind(self)
        self.socket.settimeout(1)
        self.run = True

    def get_request(self):
        while self.run:
            try:
                sock, addr = self.socket.accept()
                sock.settimeout(None)
                return (sock, addr)
            except socket.timeout:
                pass

    def stop(self):
        self.run = False

    def serve(self):
        while self.run:
            try:
                self.handle_request()
            except TypeError:
                # When server is being closed, while loop can run once
                # after setting self.run = False depending on how it
                # is scheduled.
                pass


class Server(object):
    """
    Parameters
    ----------
    port : integer
        Defines the port on which the server will run. If this port is
        already bind, then it increment 1 until it finds a free port.
    scene_file : name of the scene_file generated for visualization
        A Valid PyDy generated scene file in 'directory' parameter.
    directory : absolute path of a directory
        Absolute path to the directory which contains scene_file with
        all other static files.

    Example
    -------
    >>> server = Server(scene_file=_scene_json_file)
    >>> server.run_server()

    """
    def __init__(self, scene_file, directory="static/", port=8000):
        self.scene_file = scene_file
        self.port = port
        self.directory = directory

    def run_server(self):
        # Change dir to static first.
        os.chdir(self.directory)
        print(os.getcwd())
        # Get a free port
        while self._check_port(self.port):
            self.port += 1
        handler_class = SimpleHTTPRequestHandler
        server_class = StoppableHTTPServer
        protocol = "HTTP/1.0"
        server_address = ('127.0.0.1', self.port)
        handler_class.protocol_version = protocol
        self.httpd = server_class(server_address, handler_class)
        sa = self.httpd.socket.getsockname()
        print("Serving HTTP on", sa[0], "port", sa[1], "...")
        print("To view visualization, open:\n")
        url = "http://localhost:"+str(sa[1]) + "/index.html?load=" + \
              self.scene_file
        print(url)
        webbrowser.open(url)
        print("Hit Ctrl+C to stop the server...")
        signal.signal(signal.SIGINT, self._stop_server)
        self.httpd.serve()

    def _check_port(self, port):
        soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        result = soc.connect_ex(('127.0.0.1', port))
        return result == 0

    def _stop_server(self, signal, frame):
        """
        Confirms and stops the visulisation server
        signal:
            Required by signal.signal
        frame:
            Required by signal.signal

        """
        res = raw_input("Shutdown this visualization server ([y]/n)? ")
        res = res.lower()[0]
        if res == '' or res == 'y':
            print("Shutdown confirmed")
            print("Shutting down server...")
            self.httpd.stop()
        else:
            print("Resuming operations...")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# external libraries
import sympy as sm
import numpy as np
from numpy import testing

# local libraries
from ..code import generate_ode_function
from ...models import multi_mass_spring_damper, n_link_pendulum_on_cart

# TODO : Remove these tests before PyDy 0.4.0. They are just here to make
# sure old stuff still runs.


class TestCodeRHSArgs():

    def test_rhs_args(self):

        sys = n_link_pendulum_on_cart(3, True, True)

        constants = sys.constants_symbols
        specifieds = tuple(sys.specifieds_symbols)

        args = (sys.eom_method.mass_matrix_full,
                sys.eom_method.forcing_full,
                constants,
                sys.coordinates,
                sys.speeds)

        kwargs = {'specified': specifieds}

        rhs = generate_ode_function(*args, **kwargs)

        x = np.array(np.random.random(len(sys.coordinates + sys.speeds)))

        rhs_args = {'constants': {k: 1.0 for k in constants}}

        rhs_args['specified'] = dict(zip(specifieds, [1.0, 2.0, 3.0, 4.0]))

        xd_01 = rhs(x, 0.0, rhs_args)

        rhs_args['specified'] = {
            specifieds: lambda x, t: np.array([1.0, 2.0, 3.0, 4.0])}

        xd_02 = rhs(x, 0.0, rhs_args)

        # There are four specified inputs available.
        rhs_args['specified'] = {
            specifieds[0]: lambda x, t: np.ones(1),
            (specifieds[3], specifieds[1]): lambda x, t: np.array([4.0, 2.0]),
            specifieds[2]: 3.0 * np.ones(1)}

        xd_03 = rhs(x, 0.0, rhs_args)

        testing.assert_allclose(xd_01, xd_02)
        testing.assert_allclose(xd_01, xd_03)

        # Test old and efficient RHS args.
        rhs_args['specified'] = np.array([1.0, 2.0, 3.0, 4.0])
        xd_04 = rhs(x, 0.0, rhs_args)
        testing.assert_allclose(xd_01, xd_04)

        rhs_args['specified'] = lambda x, t: np.array([1.0, 2.0, 3.0, 4.0])
        xd_05 = rhs(x, 0.0, rhs_args)
        testing.assert_allclose(xd_01, xd_05)


class TestCode():

    def test_generate_ode_function(self):

        system = multi_mass_spring_damper(1, True, True)

        args = (system.eom_method.mass_matrix_full,
                system.eom_method.forcing_full,
                system.constants_symbols,
                system.coordinates,
                system.speeds)

        kwargs = {'specified': tuple(system.specifieds_symbols)}

        mass_matrix, forcing_vector, constants, coordinates, speeds = args
        specifieds = kwargs['specified']

        F, x, v = np.random.random(3)

        states = np.array([x, v])

        constants_map = dict(zip(constants, np.random.random(len(constants))))

        m = constants_map[sm.symbols('m0')]
        k = constants_map[sm.symbols('k0')]
        c = constants_map[sm.symbols('c0')]
        g = constants_map[sm.symbols('g')]

        expected_dx = np.array([v, 1.0 / m * (-c * v + m * g - k * x + F)])

        rhs_args = {'constants': constants_map,
                    'specified': {specifieds[0]: F}}

        backends = ['lambdify', 'theano', 'cython']

        for backend in backends:
            rhs = generate_ode_function(mass_matrix, forcing_vector,
                                        constants, coordinates, speeds,
                                        specifieds, generator=backend)
            dx = rhs(states, 0.0, rhs_args)

            testing.assert_allclose(dx, expected_dx)

        # Now try it with a function defining the specified quantities.
        rhs_args['specified'] = {specifieds[0]: lambda x, t: np.sin(t)}

        t = 14.345

        expected_dx = np.array([v, 1.0 / m * (-c * v + m * g - k * x +
                                              np.sin(t))])

        for backend in backends:
            rhs = generate_ode_function(mass_matrix, forcing_vector,
                                        constants, coordinates, speeds,
                                        specifieds, generator=backend)
            dx = rhs(states, t, rhs_args)

            testing.assert_allclose(dx, expected_dx)

        # Now try it without specified values.
        system = multi_mass_spring_damper(1, True)

        args = (system.eom_method.mass_matrix_full,
                system.eom_method.forcing_full,
                system.constants_symbols,
                system.coordinates,
                system.speeds)

        mass_matrix, forcing_vector, constants, coordinates, speeds = args

        expected_dx = np.array([v, 1.0 / m * (-c * v + m * g - k * x)])

        for backend in backends:
            rhs = generate_ode_function(mass_matrix, forcing_vector,
                                        constants, coordinates, speeds,
                                        specifieds, generator=backend)
            dx = rhs(states, 0.0, rhs_args)

            testing.assert_allclose(dx, expected_dx)



import warnings
from sympy.matrices.expressions import Identity

from ..utils import PyDyUserWarning
from .visualization_frame import VisualizationFrame

__all__ = ['PerspectiveCamera', 'OrthoGraphicCamera']

warnings.simplefilter('once', PyDyUserWarning)


class PerspectiveCamera(VisualizationFrame):
    """Creates a Perspective Camera for visualization. The camera is
    inherited from VisualizationFrame. It can be attached to dynamics
    objects, hence we can get a moving camera. All the transformation matrix
    generation methods are applicable to a Perspective Camera.

    Like VisualizationFrame, it can also be initialized using:

       1. Rigidbody
       2. ReferenceFrame, Point
       3. ReferenceFrame, Particle

    Either one of these must be supplied during initialization. Unlike
    VisualizationFrame, it does not require a Shape argument.

    Parameters
    ==========
    name : str
        A name for the PerspectiveCamera(optional). Default is 'unnamed'
    fov : float, default=45.0
        Field Of View, It determines the angle between the top and bottom of
        the viewable area (in degrees).
    near : float
        The distance of near plane of the PerspectiveCamera. All objects
        closer to this distance are not displayed.
    far : int or float
        The distance of far plane of the PerspectiveCamera. All objects
        farther than this distance are not displayed.

    """

    def __init__(self, *args, **kwargs):
        """Initialises a PerspectiveCamera object. To initialize a
        visualization frame, one needs to supply a name (optional), a
        reference frame, a point, field of view (fov) (optional), near plane
        distance (optional) and far plane distance (optional).

        Examples
        ========
        >>> from pydy.viz import VisualizationFrame, Shape
        >>> from sympy.physics.mechanics import (ReferenceFrame, Point,
        ...                                      RigidBody, Particle,
        ...                                      inertia)
        >>> from sympy import symbols
        >>> I = ReferenceFrame('I')
        >>> O = Point('O')
        >>> shape = Shape()
        >>> # initializing with reference frame, point
        >>> camera1 = PerspectiveCamera('frame1', I, O)
        >>> Ixx, Iyy, Izz, mass = symbols('Ixx Iyy Izz mass')
        >>> i = inertia(I, Ixx, Iyy, Izz)
        >>> rbody = RigidBody('rbody', O, I, mass, (inertia, O))
        >>> # Initializing with a rigidbody ..
        >>> camera2 = PerspectiveCamera('frame2', rbody)
        >>> Pa = Particle('Pa', O, mass)
        >>> # initializing with Particle, reference_frame ...
        >>> camera3 = PerspectiveCamera('frame3', I, Pa)
        """

        msg = ("Rotation of Perspective Camera does not work "
               "properly in the visualiser.")
        warnings.warn(msg, PyDyUserWarning)

        try:
            self._fov = kwargs['fov']
        except KeyError:
            self._fov = 45.0

        try:
            self._near = kwargs['near']
        except KeyError:
            self._near = 1.0

        try:
            self._far = kwargs['far']
        except KeyError:
            self._far = 1000.0

        # Now we use same approach as in VisualizationFrame for setting
        # reference_frame and origin
        i = 0
        # If first arg is not str, name the visualization frame 'unnamed'
        if isinstance(args[i], str):
            self._name = args[i]
            i += 1
        else:
            self._name = 'unnamed'

        try:
            self._reference_frame = args[i].get_frame()
            self._origin = args[i].get_masscenter()

        except AttributeError:
            # It is not a rigidbody, hence this arg should be a reference
            # frame
            try:
                # TODO : dcm is never used.
                dcm = args[i]._dcm_dict
                self._reference_frame = args[i]
                i += 1
            except AttributeError:
                raise TypeError('A ReferenceFrame is to be supplied '
                                'before a Particle/Point.')

            # Now next arg can either be a Particle or point
            try:
                self._origin = args[i].get_point()
            except AttributeError:
                self._origin = args[i]

        # basic thing required, transform matrix
        self._transform = Identity(4).as_mutable()

    def __str__(self):
        return 'PerspectiveCamera: ' + self._name

    def __repr__(self):
        return 'PerspectiveCamera'

    @property
    def fov(self):
        """
        attribute for Field Of view of a PerspectiveCamera.
        Default value is 45 degrees
        """
        return self._fov

    @fov.setter
    def fov(self, new_fov):
        if not isinstance(new_fov, (int, str)):
            raise TypeError('fov should be supplied in int or float')
        else:
            self._fov = new_fov

    @property
    def near(self):
        """
        attribute for Near Plane distance of a PerspectiveCamera.
        Default value is 1
        """
        return self._near

    @near.setter
    def near(self, new_near):
        if not isinstance(new_near, (int, str)):
            raise TypeError('near should be supplied in int or float')
        else:
            self._near = new_near

    @property
    def far(self):
        """
        Attribute for Far Plane distance of a PerspectiveCamera. The default
        value is ``1000.0``.
        """
        return self._far

    @far.setter
    def far(self, new_far):
        if not isinstance(new_far, (int, str)):
            raise TypeError('far should be supplied in int or float')
        else:
            self._far = new_far

    def generate_scene_dict(self):
        """This method generates information for a static visualization in
        the initial conditions, in the form of dictionary. This contains
        camera parameters followed by an init_orientation Key.

        Before calling this method, all the transformation matrix generation
        methods should be called, or it will give an error.

        Returns
        =======
        A dict with following Keys:

        1. name: name for the camera
        2. fov: Field of View value of the camera
        3. near: near value of the camera
        4. far: far value of the camera
        5. init_orientation: Initial orientation of the camera

        """
        scene_dict = {id(self): {}}
        scene_dict[id(self)]['name'] = self.name
        scene_dict[id(self)]['type'] = self.__repr__()
        scene_dict[id(self)]['fov'] = self.fov
        scene_dict[id(self)]['near'] = self.near
        scene_dict[id(self)]['far'] = self.far
        scene_dict[id(self)]["simulation_id"] = id(self)
        scene_dict[id(self)]["init_orientation"] = self._visualization_matrix[0]

        return scene_dict

    def generate_simulation_dict(self):
        """Generates the simulation information for this visualization
        frame.  It maps the simulation data information to the scene
        information via a unique id.

        Before calling this method, all the transformation matrix generation
        methods should be called, or it will give an error.

        Returns
        =======
        simulation_dict : dictionary
            A dictionary containing list of 4x4 matrices mapped to the
            unique id as the key.

        """
        simulation_dict = {}
        try:
            simulation_dict[id(self)] = self._visualization_matrix

        except:
            raise RuntimeError("Please call the numerical "
                               "transformation methods, "
                               "before generating visualization dict.")

        return simulation_dict


class OrthoGraphicCamera(VisualizationFrame):
    """Creates a OrthoGraphic Camera for visualization. The camera is
    inherited from ``VisualizationFrame``. It can be attached to dynamics
    objects, hence we can get a moving camera. All the transformation matrix
    generation methods are applicable to a Perspective Camera.

    Like VisualizationFrame, it can also be initialized using:

       1. :role:`Rigidbody`
       2. ReferenceFrame, Point
       3. ReferenceFrame, Particle

    Either one of these must be supplied during initialization. Unlike
    VisualizationFrame, it doesnt require a Shape argument.

    Parameters
    ==========
    name : str, optional, default='unnamed'
        A name for the PerspectiveCamera.
    near : float, optional, default=
        The distance of near plane of the PerspectiveCamera. All objects
        closer to this distance are not displayed.
    far : float, optional, default=
        The distance of far plane of the PerspectiveCamera. All objects
        farther than this distance are not displayed.

    """

    def __init__(self, *args, **kwargs):
        """
        Initialises an OrthoGraphicCamera object. To initialize a
        visualization frame, we need to supply a name (optional), a
        reference frame, a point, near plane distance (optional) and far
        plane distance (optional).

        Examples
        ========
        >>> from pydy.viz import OrthoGraphicCamera
        >>> from sympy.physics.mechanics import (ReferenceFrame, Point,
        ...                                      RigidBody, Particle,
        ...                                      inertia)
        >>> from sympy import symbols
        >>> I = ReferenceFrame('I')
        >>> O = Point('O')
        >>> shape = Shape()
        >>> # Initializing with ReferenceFrame, Point
        >>> camera1 = OrthoGraphicCamera('frame1', I, O)
        >>> Ixx, Iyy, Izz, mass = symbols('Ixx Iyy Izz mass')
        >>> i = inertia(I, Ixx, Iyy, Izz)
        >>> rbody = RigidBody('rbody', O, I, mass, (inertia, O))
        >>> # Initializing with a Rigidbody
        >>> camera2 = OrthoGraphicCamera('frame2', rbody)
        >>> Pa = Particle('Pa', O, mass)
        >>> # Initializing with Particle, ReferenceFrame
        >>> camera3 = OrthoGraphicCamera('frame3', I, Pa)
        """
        try:
            self._near = kwargs['near']
        except KeyError:
            self._near = 1

        try:
            self._far = kwargs['far']
        except KeyError:
            self._far = 1000

        # Now we use same approach as in VisualizationFrame for setting
        # reference_frame and origin
        i = 0
        # If first arg is not str, name the visualization frame 'unnamed'
        if isinstance(args[i], str):
            self._name = args[i]
            i += 1
        else:
            self._name = 'unnamed'

        try:
            self._reference_frame = args[i].get_frame()
            self._origin = args[i].get_masscenter()

        except AttributeError:
            # It is not a rigidbody, hence this arg should be a reference
            # frame.
            self._reference_frame = args[i]
            i += 1

            # Now next arg can either be a Particle or point
            try:
                self._origin = args[i].get_point()
            except AttributeError:
                self._origin = args[i]

        # basic thing required, transform matrix
        self._transform = Identity(4).as_mutable()

    def __str__(self):
        return 'OrthoGraphicCamera: ' + self._name

    def __repr__(self):
        return 'OrthoGraphicCamera'

    @property
    def near(self):
        """Attribute for Near Plane distance of an OrthoGraphicCamera.
        Default value is 1.0
        """
        return self._near

    @near.setter
    def near(self, new_near):
        if not isinstance(new_near, (int, str)):
            raise TypeError('near should be supplied in int or float')
        else:
            self._near = new_near

    @property
    def far(self):
        """Attribute for Far Plane distance of an OrthoGraphicCamera.
        Default value is 1000.0.
        """
        return self._far

    @far.setter
    def far(self, new_far):
        if not isinstance(new_far, (int, str)):
            raise TypeError('far should be supplied in int or float')
        else:
            self._far = new_far

    def generate_scene_dict(self):
        """
        This method generates information for a static visualization in the
        initial conditions, in the form of dictionary. This contains camera
        parameters followed by an init_orientation Key.

        Returns
        =======
        scene_dict : dictionary
           A dict with following Keys:

              1. name: name for the camera
              2. near: near value of the camera
              3. far: far value of the camera
              4. init_orientation: Initial orientation of the camera

        """
        scene_dict = {id(self): {}}
        scene_dict[id(self)]['name'] = self.name
        scene_dict[id(self)]['type'] = self.__repr__()
        scene_dict[id(self)]['near'] = self.near
        scene_dict[id(self)]['far'] = self.far
        scene_dict[id(self)]["simulation_id"] = id(self)
        scene_dict[id(self)]["init_orientation"] = self._visualization_matrix[0]

        return scene_dict

    def generate_simulation_dict(self):
        """Generates the simulation information for this visualization
        frame.  It maps the simulation data information to the scene
        information via a unique id.

        Returns
        =======

        A dictionary containing list of 4x4 matrices mapped to the unique id
        as the key.

        """
        simulation_dict = {}
        try:
            simulation_dict[id(self)] = self._visualization_matrix

        except:
            raise RuntimeError("Please call the numerical ",
                               "transformation methods, ",
                               "before generating visualization dict")

        return simulation_dict



#!/usr/bin/env python

import textwrap

from pkg_resources import parse_version
from setuptools import __version__ as SETUPTOOLS_VERSION
import sympy as sm
from sympy.core.function import AppliedUndef
from sympy.utilities.iterables import iterable
from sympy.physics.mechanics import dynamicsymbols

SYMPY_VERSION = sm.__version__


def sympy_equal_to_or_newer_than(version, installed_version=None):
    """Returns true if the installed version of SymPy is equal to or newer
    than the provided version string."""

    if installed_version is None:
        v = SYMPY_VERSION
    else:
        v = installed_version

    if v.endswith('-git') and \
            parse_version(SETUPTOOLS_VERSION) >= parse_version('8.0'):

        msg = ('You are using an older development version of SymPy with a '
               'non-PEP440 compliant version number: {}. Please install '
               'setuptools < 8.0 or a newer development version of SymPy.')
        raise ValueError(msg.format(v))

    return parse_version(v) >= parse_version(version)


def wrap_and_indent(lines, indentation=4, width=79):
    """Returns a single string in which the lines have been indented and
    wrapped into a block of text."""
    # TODO : This will indent any lines that only contain a new line. Which
    # may not be preferable.
    new_lines = []
    for line in lines:
        if line != '\n':
            wrapped = textwrap.wrap(line, width=width-indentation)
        else:
            wrapped = [line]
        new_lines += wrapped
    spacer = '\n' + ' ' * indentation
    return ' ' * indentation + spacer.join(new_lines)


# This is a copy of the function in SymPy. It doesn't exist in SymPy 0.7.4
# so we keep it here for now.
def find_dynamicsymbols(expression, exclude=None):
    """Find all dynamicsymbols in expression.

    >>> from sympy.physics.mechanics import dynamicsymbols, find_dynamicsymbols
    >>> x, y = dynamicsymbols('x, y')
    >>> expr = x + x.diff()*y
    >>> find_dynamicsymbols(expr)
    set([x(t), y(t), Derivative(x(t), t)])

    If the optional ``exclude`` kwarg is used, only dynamicsymbols
    not in the iterable ``exclude`` are returned.

    >>> find_dynamicsymbols(expr, [x, y])
    set([Derivative(x(t), t)])
    """
    t_set = set([dynamicsymbols._t])
    if exclude:
        if iterable(exclude):
            exclude_set = set(exclude)
        else:
            raise TypeError("exclude kwarg must be iterable")
    else:
        exclude_set = set()
    return set([i for i in expression.atoms(AppliedUndef, sm.Derivative) if
                i.free_symbols == t_set]) - exclude_set


class PyDyDeprecationWarning(DeprecationWarning):
    pass

class PyDyImportWarning(ImportWarning):
    pass

class PyDyFutureWarning(FutureWarning):
    pass

class PyDyUserWarning(UserWarning):
    pass



#!/usr/bin/env python

# external
from sympy import sin, cos, symbols
from sympy.physics.mechanics import (dynamicsymbols, ReferenceFrame, Point,
                                     RigidBody, Particle, inertia)
from numpy import radians
from numpy.testing import assert_allclose

# local
from ..shapes import Cylinder
from ..visualization_frame import VisualizationFrame
from ..camera import PerspectiveCamera, OrthoGraphicCamera
from ..scene import Scene
from ..light import PointLight


class TestVisualizationFrameScene(object):

    def __init__(self):
        #We define some quantities required for tests here..
        self.p = dynamicsymbols('p:3')
        self.q = dynamicsymbols('q:3')
        self.dynamic = list(self.p) + list(self.q)
        self.states = [radians(45) for x in self.p] + \
                               [radians(30) for x in self.q]

        self.I = ReferenceFrame('I')
        self.A = self.I.orientnew('A', 'space', self.p, 'XYZ')
        self.B = self.A.orientnew('B', 'space', self.q, 'XYZ')

        self.O = Point('O')
        self.P1 = self.O.locatenew('P1', 10 * self.I.x + \
                                      10 * self.I.y + 10 * self.I.z)
        self.P2 = self.P1.locatenew('P2', 10 * self.I.x + \
                                    10 * self.I.y + 10 * self.I.z)

        self.point_list1 = [[2, 3, 1], [4, 6, 2], [5, 3, 1], [5, 3, 6]]
        self.point_list2 = [[3, 1, 4], [3, 8, 2], [2, 1, 6], [2, 1, 1]]

        self.shape1 = Cylinder(1.0, 1.0)
        self.shape2 = Cylinder(1.0, 1.0)


        self.Ixx, self.Iyy, self.Izz = symbols('Ixx Iyy Izz')
        self.mass = symbols('mass')
        self.parameters = [self.Ixx, self.Iyy, self.Izz, self.mass]
        self.param_vals = [0, 0, 0, 0]

        self.inertia = inertia(self.A, self.Ixx, self.Iyy, self.Izz)

        self.rigid_body = RigidBody('rigid_body1', self.P1, self.A, \
                                 self.mass, (self.inertia, self.P1))

        self.global_frame1 = VisualizationFrame('global_frame1', \
                                self.A, self.P1, self.shape1)

        self.global_frame2 = VisualizationFrame('global_frame2', \
                                self.B, self.P2, self.shape2)

        self.scene1 = Scene(self.I, self.O, \
                            (self.global_frame1, self.global_frame2), \
                                             name='scene')

        self.particle = Particle('particle1', self.P1, self.mass)

        #To make it more readable
        p = self.p
        q = self.q
        #Here is the dragon ..
        self.transformation_matrix = \
            [[cos(p[1])*cos(p[2]), sin(p[2])*cos(p[1]), -sin(p[1]), 0], \
             [sin(p[0])*sin(p[1])*cos(p[2]) - sin(p[2])*cos(p[0]), \
                  sin(p[0])*sin(p[1])*sin(p[2]) + cos(p[0])*cos(p[2]), \
                  sin(p[0])*cos(p[1]), 0], \
             [sin(p[0])*sin(p[2]) + sin(p[1])*cos(p[0])*cos(p[2]), \
                 -sin(p[0])*cos(p[2]) + sin(p[1])*sin(p[2])*cos(p[0]), \
                  cos(p[0])*cos(p[1]), 0], \
             [10, 10, 10, 1]]


    def test_vframe_with_rframe(self):
        self.frame1 = VisualizationFrame('frame1', self.I, self.O, \
                                                self.shape1)

        assert self.frame1.name == 'frame1'
        assert self.frame1.reference_frame == self.I
        assert self.frame1.origin == self.O
        assert self.frame1.shape is self.shape1

        self.frame1.name = 'frame1_'
        assert self.frame1.name == 'frame1_'

        self.frame1.reference_frame = self.A
        assert self.frame1.reference_frame == self.A

        self.frame1.origin = self.P1
        assert self.frame1.origin == self.P1

        self.frame1.shape = self.shape2
        assert self.frame1.shape is self.shape2

        assert self.frame1.generate_transformation_matrix(self.I, self.O).tolist() == \
                                             self.transformation_matrix


    def test_vframe_with_rbody(self):
        self.frame2 = VisualizationFrame('frame2', self.rigid_body, \
                                                self.shape1)

        assert self.frame2.name == 'frame2'
        assert self.frame2.reference_frame == self.A
        assert self.frame2.origin == self.P1
        assert self.frame2.shape == self.shape1

        self.frame2.name = 'frame2_'
        assert self.frame2.name == 'frame2_'

        self.frame2.reference_frame = self.B
        assert self.frame2.reference_frame == self.B

        self.frame2.origin = self.P2
        assert self.frame2.origin == self.P2

        self.frame2.shape = self.shape2
        assert self.frame2.shape is self.shape2

        self.frame2.reference_frame = self.A
        self.frame2.origin = self.P1
        assert self.frame2.generate_transformation_matrix(self.I, self.O).tolist() == \
                                            self.transformation_matrix



    def test_vframe_with_particle(self):

        self.frame3 = VisualizationFrame('frame3', \
                                          self.A, self.particle, \
                                                self.shape1)

        assert self.frame3.name == 'frame3'
        assert self.frame3.reference_frame == self.A
        assert self.frame3.origin == self.P1
        assert self.frame3.shape is self.shape1

        self.frame3.name = 'frame3_'
        assert self.frame3.name == 'frame3_'

        self.frame3.reference_frame = self.B
        assert self.frame3.reference_frame == self.B

        self.frame3.origin = self.P2
        assert self.frame3.origin == self.P2

        self.frame3.shape = self.shape2
        assert self.frame3.shape is self.shape2

        self.frame3.reference_frame = self.A
        self.frame3.origin = self.P1
        assert self.frame3.generate_transformation_matrix(self.I, self.O).tolist() == \
                                             self.transformation_matrix

    def test_vframe_without_name(self):
        self.frame4 = VisualizationFrame(self.I, self.O, \
                                               self.shape1)

        assert self.frame4.name == 'unnamed'
        #To check if referenceframe and origin are defined
        #properly without name arg
        assert self.frame4.reference_frame == self.I
        assert self.frame4.origin == self.O
        assert self.frame4.shape is self.shape1

        self.frame4.name = 'frame1_'
        assert self.frame4.name == 'frame1_'

    def test_numeric_transform(self):
        self.list1 = [[0.5000000000000001, 0.5, \
                       -0.7071067811865475, 0.0, \
                      -0.14644660940672627, 0.8535533905932737, \
                                       0.5, 0.0, \
                      0.8535533905932737, -0.14644660940672627, \
                         0.5000000000000001, 0.0, \
                      10.0, 10.0, 10.0, 1.0]]


        self.list2 = [[-0.11518993731879767, 0.8178227645734215, \
                            -0.563823734943801, 0.0, \
                      0.1332055011661179, 0.5751927992738988, \
                            0.8070994598700584, 0.0, \
                      0.984371663956036, 0.017865313009926137, \
                          -0.17519491371464685, 0.0, \
                      20.0, 20.0, 20.0, 1.0]]



        self.global_frame1.generate_transformation_matrix(self.I, self.O)
        self.global_frame1.generate_numeric_transform_function(self.dynamic, \
                                                        self.parameters)

        assert_allclose(self.global_frame1.\
                              evaluate_transformation_matrix(self.states, \
                                self.param_vals), self.list1)

        self.global_frame2.generate_transformation_matrix(self.I, self.O)
        self.global_frame2.generate_numeric_transform_function(self.dynamic, \
                                                        self.parameters)

        assert_allclose(self.global_frame2.\
                              evaluate_transformation_matrix(self.states, \
                                self.param_vals), self.list2)


    def test_perspective_camera(self):
        #Camera is a subclass of VisualizationFrame, but without any
        #specific shape attached. We supply only ReferenceFrame,Point
        #to camera. and it inherits methods from VisualizationFrame

        #Testing with rigid-body ..
        camera = PerspectiveCamera('camera', self.rigid_body, fov=45, \
                                                 near=1, far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.A
        assert camera.origin == self.P1
        assert camera.fov == 45
        assert camera.near == 1
        assert camera.far == 1000

        #Testing with reference_frame, particle ..
        camera = PerspectiveCamera('camera', self.I, self.particle, \
                                          fov=45, near=1, far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.I
        assert camera.origin == self.P1
        assert camera.fov == 45
        assert camera.near == 1
        assert camera.far == 1000

        #Testing with reference_frame, point ..
        camera = PerspectiveCamera('camera', self.I, self.O, \
                                          fov=45, near=1, far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.I
        assert camera.origin == self.O
        assert camera.fov == 45
        assert camera.near == 1
        assert camera.far == 1000

        camera.name = 'camera1'
        assert camera.name == 'camera1'
        assert camera.__str__() == 'PerspectiveCamera: camera1'
        assert camera.__repr__() == 'PerspectiveCamera'

        camera.reference_frame = self.A
        assert camera.reference_frame == self.A

        camera.origin = self.P1
        assert camera.origin == self.P1

        camera.fov = 30
        assert camera.fov == 30

        camera.near = 10
        assert camera.near == 10

        camera.far = 500
        assert camera.far == 500

        #Test unnamed
        camera1 = PerspectiveCamera(self.I, self.O)
        assert camera1.name == 'unnamed'
        assert camera1.reference_frame == self.I
        assert camera1.origin == self.O
        assert camera1.fov == 45
        assert camera1.near == 1
        assert camera1.far == 1000

    def test_orthographic_camera(self):
        #As compared to Perspective Camera, Orthographic Camera doesnt
        #have fov, instead the left,right,top and bottom faces are
        #adjusted by the Scene width, and height

        #Testing with rigid_body
        camera = OrthoGraphicCamera('camera', self.rigid_body, \
                                               near=1, far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.A
        assert camera.origin == self.P1
        assert camera.near == 1
        assert camera.far == 1000

        #Testing with reference_frame, particle
        camera = OrthoGraphicCamera('camera', self.I, self.particle, \
                                                   near=1, far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.I
        assert camera.origin == self.P1
        assert camera.near == 1
        assert camera.far == 1000

        #Testing with reference_frame, point ...
        camera = OrthoGraphicCamera('camera', self.I, self.O, near=1, \
                                                               far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.I
        assert camera.origin == self.O
        assert camera.near == 1
        assert camera.far == 1000

        camera.name = 'camera1'
        assert camera.name == 'camera1'
        assert camera.__str__() == 'OrthoGraphicCamera: camera1'
        assert camera.__repr__() == 'OrthoGraphicCamera'

        camera.reference_frame = self.A
        assert camera.reference_frame == self.A

        camera.origin = self.P1
        assert camera.origin == self.P1

        camera.near = 10
        assert camera.near == 10

        camera.far = 500
        assert camera.far == 500

        camera1 = OrthoGraphicCamera(self.I, self.O)
        assert camera1.name == 'unnamed'
        assert camera1.reference_frame == self.I
        assert camera1.origin == self.O
        assert camera1.near == 1
        assert camera1.far == 1000

    def test_point_light(self):
        #Testing with rigid-body ..
        light = PointLight('light', self.rigid_body, color='blue')


        assert light.name == 'light'
        assert light.reference_frame == self.A
        assert light.origin == self.P1
        assert light.color == 'blue'

        #Testing with reference_frame, particle ..
        light = PointLight('light', self.I, self.particle, color='blue')

        assert light.name == 'light'
        assert light.reference_frame == self.I
        assert light.origin == self.P1
        assert light.color == 'blue'

        #Testing with reference_frame, point ..
        light = PointLight('light', self.I, self.O, color='blue')

        assert light.name == 'light'
        assert light.reference_frame == self.I
        assert light.origin == self.O
        assert light.color == 'blue'


        light.name = 'light1'
        assert light.name == 'light1'
        assert light.__str__() == 'PointLight: light1'
        assert light.__repr__() == 'PointLight'

        light.reference_frame = self.A
        assert light.reference_frame == self.A

        light.origin = self.P1
        assert light.origin == self.P1

        light.color = 'red'
        assert light.color == 'red'


        #Test unnamed
        light1 = PointLight(self.I, self.O)
        assert light1.name == 'unnamed'
        assert light1.reference_frame == self.I
        assert light1.origin == self.O
        assert light1.color == 'white'

    def test_scene_init(self):
        self.scene2 = Scene(self.I, self.O, \
                            self.global_frame1, self.global_frame2, \
                                            name='scene')

        assert self.scene2.name == 'scene'
        assert self.scene2.reference_frame == self.I
        assert self.scene2.origin == self.O
        assert self.scene2.visualization_frames[0] is self.global_frame1
        assert self.scene2.visualization_frames[1] is self.global_frame2

        self.scene2.name = 'scene1'
        assert self.scene2.name == 'scene1'

        self.scene2.reference_frame = self.A
        assert self.scene2.reference_frame == self.A



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

from setuptools import setup, find_packages

exec(open('pydy/version.py').read())

# I was getting the same error as:
# https://github.com/statsmodels/statsmodels/issues/1073, so the following
# line is added.
os.environ["MPLCONFIGDIR"] = "."

if sys.version_info >= (3, 0):
    NUMPY_MIN_VER = '1.9'
    SCIPY_MIN_VER = '0.14.0'
    SYMPY_MIN_VER = '0.7.5'
    CYTHON_MIN_VER = '0.20.1'
    THEANO_MIN_VER = '0.7.0'
else:
    NUMPY_MIN_VER = '1.7'
    SCIPY_MIN_VER = '0.11'
    SYMPY_MIN_VER = '0.7.4.1'
    CYTHON_MIN_VER = '0.17'
    THEANO_MIN_VER = '0.6.0'

setup(
    name='pydy',
    version=__version__,
    author='PyDy Authors',
    author_email='pydy@googlegroups.com',
    url="http://pydy.org",
    description='Python tool kit for multi-body dynamics.',
    long_description=open('README.rst').read(),
    keywords="multibody dynamics",
    license='LICENSE.txt',
    packages=find_packages(),
    install_requires=['numpy>={}'.format(NUMPY_MIN_VER),
                      'scipy>={}'.format(SCIPY_MIN_VER),
                      'sympy>={}'.format(SYMPY_MIN_VER),
                      ],
    extras_require={'doc': ['sphinx', 'numpydoc'],
                    'codegen': ['Cython>={}'.format(CYTHON_MIN_VER),
                                'Theano>={}'.format(THEANO_MIN_VER)],
                    'examples': ['matplotlib>=0.99',
                                 'ipython[notebook]>=0.3.0'],
                    },
    tests_require=['nose>=1.3.0'],
    test_suite='nose.collector',
    scripts=['bin/benchmark_pydy_code_gen.py'],
    include_package_data=True,
    classifiers=['Development Status :: 4 - Beta',
                 'Intended Audience :: Science/Research',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python :: 2.7',
                 'Topic :: Scientific/Engineering :: Physics',
                 ],
)



#!/usr/bin/env python

import numpy as np
from numpy import testing
import sympy as sm
from sympy.physics.mechanics import dynamicsymbols
from scipy.integrate import odeint

from ..system import System
from ..models import multi_mass_spring_damper, n_link_pendulum_on_cart
from ..utils import sympy_equal_to_or_newer_than

SYMPY_VERSION = sm.__version__


class TestSystem():

    def setup(self):

        # Create a simple system with one specified quantity.
        self.sys = multi_mass_spring_damper(1, apply_gravity=True,
                                            apply_external_forces=True)
        self.specified_symbol = next(iter(self.sys.specifieds_symbols))
        self.constant_map = dict(zip(sm.symbols('m0, k0, c0, g'),
                                     [2.0, 1.5, 0.5, 9.8]))
        self.sys.specifieds = {self.specified_symbol: np.ones(1)}
        self.sys.constants = self.constant_map
        self.kane = self.sys.eom_method

        # Create a system with multiple specified quantities.
        self.kane_nlink = n_link_pendulum_on_cart(3, cart_force=True,
                                                  joint_torques=True).eom_method

    def test_init(self):

        # Check defaults for most attributes.
        # -----------------------------------
        sys = System(self.kane)

        assert (sys.constants_symbols ==
                set(sm.symbols('k0, m0, g, c0')))
        assert sys.specifieds_symbols == {self.specified_symbol}
        assert sys.states == dynamicsymbols('x0, v0')
        assert sys.evaluate_ode_function is None
        assert sys.eom_method is self.kane
        assert sys.ode_solver is odeint
        assert sys.specifieds == dict()
        assert sys.initial_conditions == dict()
        assert sys.constants == dict()

        # Specify a bunch of attributes during construction.
        # --------------------------------------------------
        ic = {dynamicsymbols('x0'): 3.6, dynamicsymbols('v0'): 4.3}
        sys = System(self.kane,
                     ode_solver=odeint,
                     specifieds={self.specified_symbol: np.ones(1)},
                     initial_conditions=ic,
                     constants=self.constant_map)

        assert sys.eom_method is self.kane
        assert list(sys.specifieds.keys()) == [dynamicsymbols('f0')]
        testing.assert_allclose(list(sys.specifieds.values()),
                                [np.ones(1)])
        assert sys.initial_conditions.keys() == ic.keys()
        testing.assert_allclose(list(sys.initial_conditions.values()),
                                list(ic.values()))
        assert sys.constants.keys() == self.constant_map.keys()
        testing.assert_allclose(list(sys.constants.values()),
                                list(self.constant_map.values()))

        # Use old specifieds.
        # -------------------
        sys = System(self.kane,
                     ode_solver=odeint,
                     specifieds={'symbols': [self.specified_symbol],
                                 'values': np.ones(1)},
                     initial_conditions=ic,
                     constants=self.constant_map)

    def test_coordinates(self):
        if sympy_equal_to_or_newer_than('0.7.6'):
            assert self.sys.coordinates == self.kane.q[:]
        else:
            assert self.sys.coordinates == self.kane._q

    def test_speeds(self):
        if sympy_equal_to_or_newer_than('0.7.6'):
            assert self.sys.speeds == self.kane.u[:]
        else:
            assert self.sys.speeds == self.kane._u

    def test_states(self):
        if sympy_equal_to_or_newer_than('0.7.6'):
            assert self.sys.states == self.kane.q[:] + self.kane.u[:]
        else:
            assert self.sys.states == self.kane._q + self.kane._u

    def test_constants(self):

        # User-specified numerical values for constants.
        constants = {sm.symbols('m0'): 3.0, sm.symbols('c0'): 4.5}

        # The System should fill in the remaining constants with the defaults.

        # Construct a new system with our constants at construction time.
        # ---------------------------------------------------------------
        sys = System(self.kane, constants=constants)

        assert sys.constants.keys() == constants.keys()
        testing.assert_allclose(list(sys.constants.values()),
                                list(constants.values()))

        # Set constants after construction.
        # ---------------------------------
        sys = System(self.kane)

        # All constants have the default value.
        assert sys.constants == dict()
        sys.constants = constants

        assert sys.constants.keys() == constants.keys()
        testing.assert_allclose(list(sys.constants.values()),
                                list(constants.values()))

        # Using the property as a dict.
        # -----------------------------
        sys = System(self.kane)
        # Modifying the dict directly does change the dict.
        sys.constants[sm.symbols('m0')] = 9.3
        assert list(sys.constants.keys()) == [sm.symbols('m0')]
        testing.assert_allclose(list(sys.constants.values()), [9.3])

        # Putting in a non-constant key does not raise exception.
        sys.constants[dynamicsymbols('v0')] = 9.8
        # Then, if we integrate, we do error-checking and we get an exception.
        sys.times = [0.0, 1.0]
        with testing.assert_raises(ValueError):
            sys.integrate()

        # Provide a constant that isn't actually a constant.
        # --------------------------------------------------
        with testing.assert_raises(ValueError):
            sys.constants = {dynamicsymbols('x0'): 1.3}
        with testing.assert_raises(ValueError):
            sys.constants = {dynamicsymbols('f0'): 1.8}

    def test_specifieds(self):

        sys = System(self.kane)
        assert sys.specifieds == dict()
        sys.specifieds = {dynamicsymbols('f0'): 5.9}
        assert list(sys.specifieds.keys()) == [dynamicsymbols('f0')]
        testing.assert_allclose(list(sys.specifieds.values()), [5.9])

        # Using the property as a dict.
        # -----------------------------
        # Modifying the dict directly does change the dict.
        sys.specifieds[dynamicsymbols('f0')] = 5.1
        assert list(sys.specifieds.keys()) == [dynamicsymbols('f0')]
        testing.assert_allclose(list(sys.specifieds.values()), [5.1])
        # Putting in a non-specified key does not raise exception.
        sys.specifieds[dynamicsymbols('v0')] = 3.5
        # Then, if we integrate, we do error-checking and we get an exception.
        sys.times = [0.0, 1.0]
        with testing.assert_raises(ValueError):
            sys.integrate()

        sys = System(self.kane)
        # Putting in a value of the wrong length does not raise exception.
        sys.specifieds[dynamicsymbols('f0')] = 3.1 * np.ones(2)
        # Then, if we integrate, we do error-checking and we get an exception.
        # TODO actually, this does not seem to throw an exception.
        # TODO with testing.assert_raises(ValueError):
        # TODO     sys.integrate([0.0, 1.0])

        # The specified symbol must exist in the equations of motion and not
        # be a state.
        # ------------------------------------------------------------------
        sys = System(self.kane)
        with testing.assert_raises(ValueError):
            sys.specifieds = {sm.symbols('m0'): 5.4}
        with testing.assert_raises(ValueError):
            sys.specifieds = {dynamicsymbols('x0'): 5.1}

        # Complex error-checking when using property as a dict.
        # -----------------------------------------------------
        sys = System(self.kane_nlink)
        spec_syms = list(sys.specifieds_symbols)
        times = np.linspace(0, 0.5, 10)
        sys.specifieds = {
            spec_syms[0]: lambda x, t: np.ones(t),
            (spec_syms[3], spec_syms[1]): lambda x, t: np.array([4, 2]),
            spec_syms[2]: 3.0 * np.ones(1)}
        # These won't throw an exception b/c we're modifying the dict directly.
        sys.specifieds[spec_syms[1]] = 7.1
        sys.times = times
        # This does.
        with testing.assert_raises(ValueError):
            sys.integrate()

        sys = System(self.kane_nlink)
        # This puts too many entries in the dict.
        sys.specifieds[spec_syms[0]] = 3.7
        sys.specifieds[(spec_syms[0], spec_syms[3])] = 5.8
        sys.times = times
        with testing.assert_raises(ValueError):
            sys.integrate()

        # This gets rid of the previous default entries, and should work
        # properly.
        sys.specifieds.pop(spec_syms[0])
        sys.integrate()

        # Test old way of providing specifieds.
        # -------------------------------------
        sys = System(self.kane_nlink)
        spec_syms = list(sys.specifieds_symbols)
        # Get numbers using the new way.
        sys.specifieds = dict(zip(spec_syms, [1.0, 2.0, 3.0, 4.0]))
        sys.times = times
        x_01 = sys.integrate()

        # Now use the old way.
        sys.specifieds = {'symbols': spec_syms,
                          'values': [1.0, 2.0, 3.0, 4.0]}
        x_02 = sys.integrate()
        testing.assert_allclose(x_01, x_02)

        # Error checks for the new way.
        # -----------------------------
        with testing.assert_raises(ValueError):
            sys.specifieds = {'symbols': [sm.symbols('m1')], 'values': [1.0]}
        with testing.assert_raises(ValueError):
            sys.specifieds = {'symbols': [sm.symbols('T2, T2')],
                              'values': [1, 2]}
        with testing.assert_raises(ValueError):
            sys.specifieds = {'symbols': [dynamicsymbols('T2')],
                              'values': [1.0]}

        # Reordering causes issues!
        # -------------------------
        sys.specifieds = {
            'symbols': [spec_syms[1], spec_syms[0], spec_syms[2],
                        spec_syms[3]],
            'values': [2.0, 1.0, 3.0, 4.0]}
        # I tested: x_01 is not allclose to x_03.

        sys.generate_ode_function()
        x_04 = sys.integrate()
        testing.assert_allclose(x_01, x_04)

        # Test with no specifieds.
        sys = multi_mass_spring_damper(1, apply_gravity=True)
        sys.initial_conditions = {dynamicsymbols('x0'): 0.1,
                                  dynamicsymbols('v0'): -1.0}
        sys.times = times
        sys.integrate()

    def test_ode_solver(self):

        assert self.sys.ode_solver == odeint
        self.sys.ode_solver = max
        assert self.sys.ode_solver is max

        # ode_solver must be a function
        # -----------------------------
        with testing.assert_raises(ValueError):
            self.sys.ode_solver = 5

    def test_initial_conditions(self):

        # Partially provided ic's.
        ic = {dynamicsymbols('v0'): 6.1}

        # Using the constructor.
        # ----------------------
        sys = System(self.kane, initial_conditions=ic)
        assert sys.initial_conditions.keys() == ic.keys()
        testing.assert_allclose(list(sys.initial_conditions.values()),
                                list(ic.values()))

        # Set the attribute.
        # ------------------
        sys = System(self.kane)
        sys.initial_conditions = ic
        assert sys.initial_conditions.keys() == ic.keys()
        testing.assert_allclose(list(sys.initial_conditions.values()),
                                list(ic.values()))

        # Using the property as a dict.
        # -----------------------------
        # Modifying hte dict directly does change the dict.
        sys = System(self.kane, times=[0.0, 1.0])
        sys.initial_conditions[dynamicsymbols('x0')] = 5.8
        assert list(sys.initial_conditions.keys()) == [dynamicsymbols('x0')]
        testing.assert_allclose(list(sys.initial_conditions.values()), [5.8])
        # Putting in a non-state key does not raise exception.
        sys.initial_conditions[sm.symbols('m0')] = 7.9
        # Then, if we integrate, we do error-checking and we get an exception.

        with testing.assert_raises(ValueError):
            sys.integrate()

        # Keys must be coords or speeds.
        # ------------------------------
        with testing.assert_raises(ValueError):
                self.sys.initial_conditions = {sm.symbols('k0'): 0.4}
        with testing.assert_raises(ValueError):
                self.sys.initial_conditions = {sm.symbols('f0'): 7.3}

    def test_generate_ode_function(self):

        rhs = self.sys.generate_ode_function()

        assert rhs is self.sys.evaluate_ode_function

        args = (self.sys.specifieds, self.sys.constants)

        actual = rhs(np.ones(2), 0.0, *args)

        # Regression.
        testing.assert_allclose(actual, np.array([1, 9.3]))

        # n-link cart: play with specifieds.
        # ----------------------------------
        sys = System(self.kane_nlink)
        spec_syms = list(sys.specifieds_symbols)
        rhs = sys.generate_ode_function()
        x = np.array(np.random.random(len(sys.states)))
        args = (self.sys.specifieds,
                {k: 1.0 for k in sys.constants_symbols})

        # Specify constants in two different ways and ensure we get the
        # same results. This is like Jason's test in codegen.
        args = (dict(zip(spec_syms, [1.0, 2.0, 3.0, 4.0])),
                {k: 1.0 for k in sys.constants_symbols})
        xd_01 = rhs(x, 0.0, *args)

        args = ({spec_syms[0]: lambda x, t: np.ones(1),
                 (spec_syms[3], spec_syms[1]): lambda x, t: np.array([4, 2]),
                 spec_syms[2]: 3.0 * np.ones(1)},
                {k: 1.0 for k in sys.constants_symbols})
        xd_02 = rhs(x, 0.0, *args)

        testing.assert_allclose(xd_01, xd_02)

    def test_integrate(self):

        times = np.linspace(0, 1, 100)

        # Try without calling generate_ode_function.
        # ------------------------------------------
        sys = System(self.kane, times=times)
        x_01 = sys.integrate()

        sys = System(self.kane, times=times)
        sys.generate_ode_function(generator='lambdify')
        x_02 = sys.integrate()

        testing.assert_allclose(x_01, x_02)

        # Ensure that the defaults are as expected.
        # -----------------------------------------
        constants_dict = dict(zip(sm.symbols('m0, k0, c0, g'),
                                  [1.0, 1.0, 1.0, 1.0]))
        specified_dict = {dynamicsymbols('f0'): 0.0}
        x_03 = sys.ode_solver(sys.evaluate_ode_function, [0, 0], sys.times,
                              args=(specified_dict, constants_dict))
        testing.assert_allclose(x_02, x_03)

        # Ensure that initial conditions are reordered properly.
        # ------------------------------------------------------
        sys = System(self.kane, times=times)
        # I know that this is the order of the states.
        x0 = [5.1, 3.7]
        ic = {dynamicsymbols('x0'): x0[0], dynamicsymbols('v0'): x0[1]}
        sys.initial_conditions = ic
        x_04 = sys.integrate()
        x_05 = sys.ode_solver(
            sys.evaluate_ode_function, x0, sys.times,
            args=(sys._specifieds_padded_with_defaults(),
                  sys._constants_padded_with_defaults()))

        testing.assert_allclose(x_04, x_05)

        # Test a generator other than lambdify.
        # -------------------------------------
        sys.generate_ode_function(generator='theano')
        sys.times = times
        x_06 = sys.integrate()
        testing.assert_allclose(x_04, x_06)

        # Unrecognized generator.
        # -----------------------
        sys = System(self.kane, times=times)
        with testing.assert_raises(NotImplementedError):
            sys.generate_ode_function(generator='made-up')



#!/usr/bin/env python

from pkg_resources import parse_version
from setuptools import __version__ as SETUPTOOLS_VERSION
from nose.tools import assert_raises

from ..utils import sympy_equal_to_or_newer_than


def test_sympy_equal_to_or_newer_than():

    # sympy_equal_to_or_newer_than(version, installed_version)
    assert sympy_equal_to_or_newer_than('0.7.6.dev', '0.7.6.dev')
    assert not sympy_equal_to_or_newer_than('0.7.6', '0.7.6.dev')
    assert sympy_equal_to_or_newer_than('0.7.5', '0.7.6.dev')
    assert sympy_equal_to_or_newer_than('0.6.5', '0.7.6.dev')
    assert not sympy_equal_to_or_newer_than('0.7.7', '0.7.6.dev')
    if parse_version(SETUPTOOLS_VERSION) >= parse_version('8.0'):
        with assert_raises(ValueError):
            sympy_equal_to_or_newer_than('0.7.7', '0.7.6-git')
